import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  map,
  merge,
  Observable,
  of,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { Casino } from '../models/casino';
import { CasinoPagination } from '../models/casino-pagination';
import { CasinoService } from '../services/casino.service';
import { ActivatedRoute } from '@angular/router';
import { ListBettingHistoryResponse } from 'app/modules/proto/c2se/api/betting_pb';
import { BettingHistoryModel } from 'app/modules/proto/models/api/betting_pb';
import { MemberModel } from 'app/modules/proto/models/member_pb';
import { ListMembersResponse } from 'app/modules/proto/c2se/member_pb';
import { environment } from 'environments/environment';

@Component({
  selector: 'casino-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .casino-grid {
        /*                     상부   유저   게임  형식  금액 */
        grid-template-columns: 100px 140px auto 100px 100px;

        @screen sm {
          /*                     상부   유저   게임  형식  금액   배팅 */
          grid-template-columns: 100px 140px auto 100px 100px 140px;
        }

        @screen md {
          /*                     상부   유저   게임  형식  금액    배팅 데이터  콤프  */
          grid-template-columns: 100px 140px auto 100px 100px 140px 100px 20px;
        }

        @screen lg {
          /*                     상부   유저   게임  형식  금액    배팅  데이터  콤프  롤링  배팅시간  */
          grid-template-columns: 100px 140px auto 100px 100px 140px 100px 20px 140px 100px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  casinos$!: Observable<Casino[] | undefined>;
  bettingHistorys$!: Observable<BettingHistoryModel[]>;

  listMember!: MemberModel[];

  mainOfficeId = environment.constants.classeIds.mainOffice;
  branchId = environment.constants.classeIds.branch;
  divisionId = environment.constants.classeIds.division;
  officeId = environment.constants.classeIds.office;
  storeId = environment.constants.classeIds.store;

  __isSearchOpened = false;
  isLoading = false;
  searchInputControl = new FormControl();
  selectedCasino?: Casino;
  pagination?: CasinoPagination;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _activatedRoute: ActivatedRoute,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _casinoService: CasinoService
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._casinoService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: CasinoPagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products
    this.casinos$ = this._casinoService.casinos$;

    // Get the products
    this._activatedRoute.data.subscribe((data) => {
      let listBettingHistoryResult: ListBettingHistoryResponse.Result =
        data['listBettingHistory'];
      let listMemberResult: ListMembersResponse.Result = data['listMember'];

      this.listMember = listMemberResult.getMembersList();

      const listBettingHistory =
        listBettingHistoryResult.getBettingHistoryList();
      this.bettingHistorys$ = of(listBettingHistory);

      // Mark for check
      this._changeDetectorRef.markForCheck();
    });
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'signinId',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.markForCheck();

      // If the casino changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;
            return this._casinoService.getCasinos(
              this._paginator.pageIndex,
              this._paginator.pageSize,
              this._sort.active,
              this._sort.direction
            );
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    }
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * toggle the search
   * Used in 'bar'
   */
  __onClickSearch(): void {
    this.__isSearchOpened = !this.__isSearchOpened;
  }

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __getCompType(info: BettingHistoryModel): string {
    const member = this.__getMemberByUsername(info.getSiteUsername());

    const type = member?.getMemberSettlementSetting()?.getCanPerChargeComp();

    return !!type ? 'Y' : 'N';
  }

  __getMemberSettlementByParentId(
    parentId: string,
    type: string
  ): MemberModel | undefined {
    const parentMember = this.__getMemberByParentId(parentId);
    if (!!parentMember && parentMember.getMemberClass()?.getId() === type) {
      return parentMember;
    }
    return this.__getMemberSettlementByParentId(parentId, type);
  }

  __getMemberByParentId(parentId: string): MemberModel | undefined {
    const findMember = this.listMember.find((v) => v.getId() === parentId);

    return findMember;
  }
  __getMemberByUsername(username: string): MemberModel | undefined {
    const findMember = this.listMember.find(
      (v) => v.getUsername() === username
    );

    return findMember;
  }

  __getParentMemberByBettingHistory(info: BettingHistoryModel): string {
    const parentId = info.getMember()?.getParentMemberId();

    const pararent = this.listMember.find((v) => v.getId() === parentId);

    const result = `[${pararent
      ?.getMemberClass()
      ?.getName()}]${pararent?.getUsername()}`;

    return result;
  }
}
