import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { BettingService } from 'app/modules/polyglot/api/services/betting.service';
import {
  ListBettingHistoryRequest,
  ListBettingHistoryResponse,
} from 'app/modules/proto/c2se/api/betting_pb';
import { Pagination } from 'app/modules/proto/protobuf/pagination_pb';
import { environment } from 'environments/environment.prod';
import { catchError, Observable, throwError } from 'rxjs';

import { Casino } from '../models/casino';
import { CasinoPagination } from '../models/casino-pagination';
import { CasinoService } from '../services/casino.service';

@Injectable({
  providedIn: 'root',
})
export class CasinoResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _casinoService: CasinoService, private _router: Router) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<Casino | undefined> {
    return this._casinoService.getCasinoById(route.paramMap.get('id')).pipe(
      // Error here means the requested product is not available
      catchError((error) => {
        // Log the error
        console.error(error);

        // Get the parent url
        const parentUrl = state.url.split('/').slice(0, -1).join('/');

        // Navigate to there
        this._router.navigateByUrl(parentUrl);

        // Throw an error
        return throwError(error);
      })
    );
  }
}

@Injectable({
  providedIn: 'root',
})
export class CasinosResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _casinoService: CasinoService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: CasinoPagination;
    casinos: Casino[];
  }> {
    console.log('state.children: ', state.url);

    return this._casinoService.getCasinos();
  }
}

@Injectable({
  providedIn: 'root',
})
export class ListCasinosResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _bettingService: BettingService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ):
    | Observable<ListBettingHistoryResponse.Result>
    | Promise<ListBettingHistoryResponse.Result>
    | ListBettingHistoryResponse.Result {
    const url = state.url;
    const startIdx = url.lastIndexOf('/');
    const target = url.substring(startIdx + 1);

    const req = new ListBettingHistoryRequest();
    const search = new ListBettingHistoryRequest.Search();
    const pagination = new Pagination();

    let vendorIds: number[] = [];

    switch (target) {
      case 'evolution':
        vendorIds = environment.constants.venderIds.evolutionCasino;
        break;
      case 'casino':
        vendorIds = environment.constants.venderIds.casinos;
        break;
      case 'slot':
        vendorIds = environment.constants.venderIds.slots;
        break;

      case 'powerball':
        break;
      default:
        break;
    }

    search.setVendorIdsList(vendorIds);
    pagination.setPage(1);
    pagination.setPageSize(10);

    req.setSearch(search);
    req.setPagination(pagination);

    return this._bettingService.listVendors(req);
  }
}
